// ┌──────────────────────────────────────────────────────────────────────────────────────┐ \\
// │ Eve 0.3.4 - JavaScript Events Library                                                │ \\
// ├──────────────────────────────────────────────────────────────────────────────────────┤ \\
// │ Copyright (c) 2008-2011 Dmitry Baranovskiy (http://dmitry.baranovskiy.com/)          │ \\
// │ Licensed under the MIT (http://www.opensource.org/licenses/mit-license.php) license. │ \\
// └──────────────────────────────────────────────────────────────────────────────────────┘ \\

(function (glob) {
	var version = "0.3.4",
		has = "hasOwnProperty",
		separator = /[\.\/]/,
		wildcard = "*",
		fun = function () {},
		numsort = function (a, b) {
			return a - b;
		},
		current_event,
		stop,
		events = {n: {}},

		eve = function (name, scope) {
			var e = events,
				oldstop = stop,
				args = Array.prototype.slice.call(arguments, 2),
				listeners = eve.listeners(name),
				z = 0,
				f = false,
				l,
				indexed = [],
				queue = {},
				out = [],
				ce = current_event,
				errors = [];
			current_event = name;
			stop = 0;
			for (var i = 0, ii = listeners.length; i < ii; i++) if ("zIndex" in listeners[i]) {
				indexed.push(listeners[i].zIndex);
				if (listeners[i].zIndex < 0) {
					queue[listeners[i].zIndex] = listeners[i];
				}
			}
			indexed.sort(numsort);
			while (indexed[z] < 0) {
				l = queue[indexed[z++]];
				out.push(l.apply(scope, args));
				if (stop) {
					stop = oldstop;
					return out;
				}
			}
			for (i = 0; i < ii; i++) {
				l = listeners[i];
				if ("zIndex" in l) {
					if (l.zIndex == indexed[z]) {
						out.push(l.apply(scope, args));
						if (stop) {
							break;
						}
						do {
							z++;
							l = queue[indexed[z]];
							l && out.push(l.apply(scope, args));
							if (stop) {
								break;
							}
						} while (l)
					} else {
						queue[l.zIndex] = l;
					}
				} else {
					out.push(l.apply(scope, args));
					if (stop) {
						break;
					}
				}
			}
			stop = oldstop;
			current_event = ce;
			return out.length ? out : null;
		};

	eve.listeners = function (name) {
		var names = name.split(separator),
			e = events,
			item,
			items,
			k,
			i,
			ii,
			j,
			jj,
			nes,
			es = [e],
			out = [];
		for (i = 0, ii = names.length; i < ii; i++) {
			nes = [];
			for (j = 0, jj = es.length; j < jj; j++) {
				e = es[j].n;
				items = [e[names[i]], e[wildcard]];
				k = 2;
				while (k--) {
					item = items[k];
					if (item) {
						nes.push(item);
						out = out.concat(item.f || []);
					}
				}
			}
			es = nes;
		}
		return out;
	};


	eve.on = function (name, f) {
		var names = name.split(separator),
			e = events;
		for (var i = 0, ii = names.length; i < ii; i++) {
			e = e.n;
			!e[names[i]] && (e[names[i]] = {n: {}});
			e = e[names[i]];
		}
		e.f = e.f || [];
		for (i = 0, ii = e.f.length; i < ii; i++) if (e.f[i] == f) {
			return fun;
		}
		e.f.push(f);
		return function (zIndex) {
			if (+zIndex == +zIndex) {
				f.zIndex = +zIndex;
			}
		};
	};

	eve.stop = function () {
		stop = 1;
	};

	eve.nt = function (subname) {
		if (subname) {
			return new RegExp("(?:\\.|\\/|^)" + subname + "(?:\\.|\\/|$)").test(current_event);
		}
		return current_event;
	};


	eve.off = eve.unbind = function (name, f) {
		var names = name.split(separator),
			e,
			key,
			splice,
			i, ii, j, jj,
			cur = [events];
		for (i = 0, ii = names.length; i < ii; i++) {
			for (j = 0; j < cur.length; j += splice.length - 2) {
				splice = [j, 1];
				e = cur[j].n;
				if (names[i] != wildcard) {
					if (e[names[i]]) {
						splice.push(e[names[i]]);
					}
				} else {
					for (key in e) if (e[has](key)) {
						splice.push(e[key]);
					}
				}
				cur.splice.apply(cur, splice);
			}
		}
		for (i = 0, ii = cur.length; i < ii; i++) {
			e = cur[i];
			while (e.n) {
				if (f) {
					if (e.f) {
						for (j = 0, jj = e.f.length; j < jj; j++) if (e.f[j] == f) {
							e.f.splice(j, 1);
							break;
						}
						!e.f.length && delete e.f;
					}
					for (key in e.n) if (e.n[has](key) && e.n[key].f) {
						var funcs = e.n[key].f;
						for (j = 0, jj = funcs.length; j < jj; j++) if (funcs[j] == f) {
							funcs.splice(j, 1);
							break;
						}
						!funcs.length && delete e.n[key].f;
					}
				} else {
					delete e.f;
					for (key in e.n) if (e.n[has](key) && e.n[key].f) {
						delete e.n[key].f;
					}
				}
				e = e.n;
			}
		}
	};

	eve.once = function (name, f) {
		var f2 = function () {
			var res = f.apply(this, arguments);
			eve.unbind(name, f2);
			return res;
		};
		return eve.on(name, f2);
	};

	eve.version = version;
	eve.toString = function () {
		return "You are running Eve " + version;
	};
	(typeof module != "undefined" && module.exports) ? (module.exports = eve) : (typeof define != "undefined" ? (define("eve", [], function() { return eve; })) : (glob.eve = eve));
})(this);